*+--------------------------------------------------------------------
*+
*+ Source Module => c:\hmg.3.4.4\0\DDLABEL\\OL2DBF.PRG
*+
*+    Copyright(C) 1983-2020 by Auge & Ohr
*+
*+    Functions: Function HB_GTSYS()
*+               Procedure MAIN()
*+               Procedure Fillit()
*+               Function MyErrorHandler()
*+               Static Function CRE_OUTLOOK()
*+               Static Function CRE_INDEX()
*+               Static Function LogMe()
*+               Static Function AppName()
*+               Static Function StripTime()
*+
*+       Tables: USE ( zDBF ) EXCLUSIVE CODEPAGE "DEWIN"
*+               USE (zDBF) EXCLUSIVE
*+
*+      Indexes: INDEX ON EVENTID TAG "ID" TO &( _cdxname )
*+               INDEX ON DTOS( STARTDATE ) TAG "MYSTART" TO &( _cdxname )
*+               INDEX ON SUBJECT TAG "SUBJECT" TO &( _cdxname )
*+               INDEX ON IF( MARKDEL, 'Y', 'N' ) TAG "MARKDEL" TO &( _cdxname )
*+
*+    Reformatted by Click! 2.05.36 on Nov-24-2020 at  6:53 am
*+
*+--------------------------------------------------------------------

#include "inkey.ch"
#include "common.ch"

#DEFINE olFolderCalendar   9

*+--------------------------------------------------------------------
*+
*+    Function HB_GTSYS()
*+
*+--------------------------------------------------------------------
*+
FUNCTION HB_GTSYS()

   REQUEST DBFCDX
   REQUEST HB_LANG_DEWIN
   REQUEST HB_CODEPAGE_DEWIN
   REQUEST HB_GT_WIN_DEFAULT                                          // Console
RETURN NIL

*+--------------------------------------------------------------------
*+
*+    Procedure MAIN()
*+
*+--------------------------------------------------------------------
*+
PROCEDURE MAIN

LOCAL cAppPath := LEFT( APPNAME( .t. ), LEN( APPNAME( .t. ) ) - LEN( APPNAME( .f. ) ) )
LOCAL zDBF     := cAppPath + "OUTLOOK.DBF"
LOCAL cStart
LOCAL cEnd
LOCAL nMax     := 25                                                  // MAXROW()
LOCAL nCol     := 80                                                  // MAXCOL()
LOCAL Getlist  := {}

   RDDSETDEFAULT( "DBFCDX" )

   SET EPOCH TO YEAR( DATE() ) - 50
   SET CENTURY ON
   SET DATE GERMAN

   hb_cdpSelect( 'DEWIN' )
   hb_langSelect( 'DE' )

   cStart := "01.01.1980"
   cEnd := DTOC( DATE() )

   // create DBF
   //
   IF !FILE( zDBF )
      CRE_OUTLOOK( zDBF )
   ENDIF

   // Office COM seems to use ANSI not UTF8
   USE ( zDBF ) EXCLUSIVE CODEPAGE "DEWIN"
   ZAP

   CLS
   // @ (nMax/2)-1,(nCol/2) SAY "Show from "
   // @ (nMax/2)  ,(nCol/2) SAY "Start Date" GET cStart
   // @ (nMax/2)+1,(nCol/2) SAY "End   Date" GET cEnd
   //
   // READ
   // CLEAR GETS

   SET ALTER TO DATETIME.TXT
   Fillit( cStart, cEnd )
   SET ALTER TO

   GO TOP
   BROWSE()

RETURN

*+--------------------------------------------------------------------
*+
*+    Procedure Fillit()
*+
*+    Called from ( ol2dbf.prg )   1 - procedure main()
*+
*+--------------------------------------------------------------------
*+
PROCEDURE Fillit( cStart, cEnd )

LOCAL oOutlook
LOCAL oNameSpace
LOCAL oAppointment
LOCAL currentAppointment
LOCAL i, iMax
LOCAL oItem
LOCAL cStringStart
LOCAL cDateStart
LOCAL cTimeStart
LOCAL nRMBS
LOCAL bSaveError
LOCAL oError
LOCAL nMax               := MAXROW()
LOCAL nCol               := MAXCOL()
LOCAL cFilter
LOCAL cMax
LOCAL aDateSec
LOCAL nTimeStart
LOCAL nEndStart
LOCAL cText, cEventID
LOCAL nHit               := 0
LOCAL nMiss              := 0

   oOutlook := CreateObject( "Outlook.Application" )
   IF EMPTY( oOutLook )
      RETURN
   ENDIF

   oNameSpace := oOutlook:GetNamespace( "MAPI" )

   // cFilter      := "[Start] >= '"+ cStart + "' and [Start] <= '"+ cEnd +"'"
   cFilter := "[Start] >= '" + cStart + "'"
   oAppointment := oNameSpace:getDefaultFolder( olFolderCalendar ) :Items

   currentAppointment := oAppointment:Restrict( cFilter )

   iMax := currentAppointment:count()
   cMax := LTRIM( STR( iMax ) )

   nTimeStart := SECONDS()

   bSaveError := ERRORBLOCK( { | oError | MyErrorHandler( oError ) } )

   FOR i := 1 TO iMax
      oItem := currentAppointment:item( i )

      APPEND BLANK

      BEGIN SEQUENCE
         LogMe( "1 Start" )

         cStringStart := hb_TToC( oItem:Start )

         LogMe( "2 Start " + cStringStart )

         // GERMAN Notation
         cDateStart := SUBSTR( cStringStart, 1, 10 )                  // YYYY.MM.DDHHMMSS

         REPLACE OUTLOOK->STARTDATE WITH CTOD( cDateStart )

         cTimeStart := SUBSTR( cStringStart, 12, 12 )                 // YYYY.MM.DDHHMMSS
         REPLACE OUTLOOK->STARTTIME WITH cTimeStart

         nHit ++
      RECOVER USING oError
         nMiss ++
         cStringStart := ""
         LogMe( "0 Start                      FAIL" )
      END SEQUENCE

      @ nMax, nCol / 3 SAY cMax + "/" + STR( i ) + " " + cStringStart

      BEGIN SEQUENCE
         LogMe( "1 Start" )

         cStringStart := hb_TToC( oItem:End )

         LogMe( "2 Start " + cStringStart )

         // GERMAN Notation
         cDateStart := SUBSTR( cStringStart, 1, 10 )                  // YYYY.MM.DDHHMMSS

         REPLACE OUTLOOK->ENDDATE WITH CTOD( cDateStart )

         cTimeStart := SUBSTR( cStringStart, 12, 12 )                 // YYYY.MM.DDHHMMSS
         REPLACE OUTLOOK->ENDTIME WITH cTimeStart

         nHit ++
      RECOVER USING oError
         nMiss ++
         cStringStart := ""
         LogMe( "0 Start                      FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE
         LogMe( "1 CreationTime" )

         //  MUST !!! else "corrupt" !
         cStringStart := hb_TToC( oItem:CreationTime )

         cDateStart := SUBSTR( cStringStart, 1, 10 )
         cTimeStart := SUBSTR( cStringStart, 12, 12 )                 // YYYY.MM.DDHHMMSS

         cStringStart := DTOS( CTOD( cDateStart ) ) + StripTime( cTimeStart )

         LogMe( "2 CreationTime" + cStringStart )
         REPLACE OUTLOOK->CREDATE WITH cStringStart

         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 CreationTime               FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE

         LogMe( "1 LastModificationTime" )

         //  MUST !!! else "corrupt" !
         * cStringStart := DTOC( oItem:LastModificationTime )
         cStringStart := hb_TToC( oItem:LastModificationTime  )

         cDateStart := SUBSTR( cStringStart, 1, 10 )
         cTimeStart := SUBSTR( cStringStart, 12, 12 )                 // YYYY.MM.DDHHMMSS

         cStringStart := DTOS( CTOD( cDateStart ) ) + StripTime( cTimeStart )

         LogMe( "2 LastModificationTime" + cStringStart )
         REPLACE OUTLOOK->LASTDATE WITH cStringStart

         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 LastModificationTime       FAIL" )
      END SEQUENCE

      // "normal" Property
      BEGIN SEQUENCE

         cEventID := oItem:EntryID
         REPLACE OUTLOOK->EVENTID WITH cEventID

         // nHit++
      RECOVER USING oError
         // REPLACE OUTLOOK->EVENTID     WITH (i+1000000)
      END SEQUENCE

      BEGIN SEQUENCE

         REPLACE OUTLOOK->SUBJECT WITH oItem:Subject

         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 Subject                    FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE

         REPLACE OUTLOOK->LOCATION WITH oItem:Location

         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 Location                   FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE

         REPLACE OUTLOOK->LABELCOLOR WITH oItem:Categories

         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 Categories                 FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE

         REPLACE OUTLOOK->MEMOTEXT WITH oItem:Body

         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 Body                       FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE

         REPLACE OUTLOOK->Duration WITH oItem:Duration
         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 Duration                   FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE

         REPLACE OUTLOOK->ALLDAY WITH oItem:AllDayEvent
         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 AllDayEvent                FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE

         REPLACE OUTLOOK->RECURREN WITH oItem:RecurrenceState
         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 RecurrenceState            FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE

         nRMBS := oItem:ReminderMinutesBeforeStart

         // hm ... berlauf ...
         //
         IF nRMBS > 999
            nRMBS := 999
         ENDIF
         REPLACE OUTLOOK->WARNTIME WITH nRMBS
         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 ReminderMinutesBeforeStart FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE

         // numeric OUTLOOK.CH Constante
         REPLACE OUTLOOK->BUSYSTAT WITH oItem:BusyStatus
         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 BusyStatus                 FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE

         // numeric OUTLOOK.CH Constante
         REPLACE OUTLOOK->IMPORTAN WITH oItem:Importance
         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 Importance                 FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE

         // numeric OUTLOOK.CH Constante
         REPLACE OUTLOOK->REMINDER WITH oItem:ReminderSet
         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 ReminderSet                FAIL" )
      END SEQUENCE

      // hm ... 0
      BEGIN SEQUENCE

         // need testing
         REPLACE OUTLOOK->MEETING WITH oItem:MeetingStatus
         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 MeetingStatus              FAIL" )
      END SEQUENCE

      BEGIN SEQUENCE

         // need testing
         REPLACE OUTLOOK->PRIVATE WITH oItem:Sensitivity
         nHit ++
      RECOVER USING oError
         nMiss ++
         LogMe( "0 Sensitivity                FAIL" )
      END SEQUENCE

   NEXT

   CLS
   ? ""
   ? ""

   nEndStart := SECONDS()
   SET DECIMALS TO 4

   ? cText := STR( nEndStart - nTimeStart ) + " Sec. for " + LTRIM( STR( i ) ) + " Records"
   LogMe( cText )
   IF i > 0
      ? cText := STR( ( nEndStart - nTimeStart ) / i ) + " Records per Sec."
      LogMe( cText )

      ? cText := STR( ( nEndStart - nTimeStart ) / nHit ) + " Sec. per ActiveX Calls"
      LogMe( cText )

      ? cText := STR( nMiss ) + " ActiveX Calls fail"
      LogMe( cText )
   ENDIF

   ? ""
   ? ""
   ? ""

   BEGIN SEQUENCE
      oOutlook:Quit()
      oOutlook := NIL
   RECOVER USING oError
   END SEQUENCE

   ERRORBLOCK( bSaveError )

RETURN

*+--------------------------------------------------------------------
*+
*+    Function MyErrorHandler()
*+
*+    Called from ( ol2dbf.prg )   1 - procedure fillit()
*+
*+--------------------------------------------------------------------
*+
FUNCTION MyErrorHandler()                                             // Fehler-Routine

   BREAK
RETURN .F.

*+--------------------------------------------------------------------
*+
*+    Static Function CRE_OUTLOOK()
*+
*+    Called from ( ol2dbf.prg )   1 - procedure main()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION CRE_OUTLOOK( datei, alias, id )

LOCAL p, field_list := {}

   IF VALTYPE( datei ) != "C"
      datei := "OUTLOOK.DBF"
   ENDIF
   IF VALTYPE( alias ) != "C"
      p := AT( ".", datei )
      alias := IF( p > 0, SUBSTR( datei, 1, p - 1 ), datei )
   ENDIF
   IF VALTYPE( id ) != "N"
      id := 0
   ENDIF
   SELECT ( id )
   IF !FILE( datei )
      AADD( field_list, { "CREDATE", "C", 18, 0 } )                   // YYYYMMDDHHMMSS.ccc
      AADD( field_list, { "LASTDATE", "C", 18, 0 } )

      AADD( field_list, { "SUBJECT", "C", 100, 0 } )
      AADD( field_list, { "LOCATION", "C", 100, 0 } )
      AADD( field_list, { "STARTDATE", "D", 8, 0 } )
      AADD( field_list, { "STARTTIME", "C", 8, 0 } )
      AADD( field_list, { "ENDDATE"    , "D",   8, 0 } )
      AADD( field_list, { "ENDTIME"    , "C",   8, 0 } )
      AADD( field_list, { "DURATION", "N", 5, 0 } )
      AADD( field_list, { "ALLDAY", "L", 1, 0 } )
      AADD( field_list, { "LABELCOLOR", "C", 20, 0 } )
      //      aadd( field_list, { "LABELTEXT"  , "C",  30, 0 } )
      AADD( field_list, { "PRIVATE", "N", 3, 0 } )
      AADD( field_list, { "MEETING", "N", 3, 0 } )
      AADD( field_list, { "REMINDER", "L", 1, 0 } )
      AADD( field_list, { "WARNTIME", "N", 3, 0 } )
      AADD( field_list, { "REMINDTIME", "N", 14, 0 } )
      //      AADD( field_list, { "SCHEDULE"   , "N",   3, 0 } )
      //      aadd( field_list, { "SHOWTIMEAS" , "C",  30, 0 } )
      AADD( field_list, { "IMPORTAN", "N", 3, 0 } )
      AADD( field_list, { "BUSYSTAT", "N", 3, 0 } )
      AADD( field_list, { "RECURREN", "N", 3, 0 } )                   // RecurrenceState only

      //
      // just for mark Record as deleted
      //
      AADD( field_list, { "MARKDEL", "L", 1, 0 } )                    // is Record deleted
      //
      // Memofields
      //
      AADD( field_list, { "MEMOTEXT", "M", 10, 0 } )
      //      AADD( field_list, { "MAKEUP"     , "M",  10, 0 } )  // XML MarkUp

      AADD( field_list, { "EVENTID", "C", 48, 0 } )

      DBCREATE( datei, field_list, "DBFCDX" )

   ENDIF

RETURN .T.

*+--------------------------------------------------------------------
*+
*+    Static Function LogMe()
*+
*+    Called from ( ol2dbf.prg )  26 - procedure fillit()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION LogMe( cText )

   SET ALTER ON
   SET CONSOLE OFF

   ? cText

   SET CONSOLE ON
   SET ALTER OFF

RETURN nil

*+--------------------------------------------------------------------
*+
*+    Static Function AppName()
*+
*+    Called from ( ol2dbf.prg )   3 - procedure main()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION AppName( lWithPath )

LOCAL n
LOCAL cName := hb_argV( 0 )
   //
   DEFAULT lWithPath TO .f.
   //
   IF !( lWithPath )
      IF ( n := RAT( '\', cName ) ) > 0
         cName := SUBSTR( cName, n + 1 )
      ENDIF
   ENDIF
   //
RETURN cName

*+--------------------------------------------------------------------
*+
*+    Static Function StripTime()
*+
*+    Called from ( ol2dbf.prg )   1 - procedure fillit()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION StripTime( cTime )

LOCAL cStrip := SUBSTR( cTime, 1, 2 ) + SUBSTR( cTime, 4, 2 ) + SUBSTR( cTime, 7, 2 ) + SUBSTR( cTime, 14)
RETURN cStrip

//
//
//

*+ EOF: OL2DBF.PRG